﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

// -------------------------------------------------------
// CLASS	:	AudioCue
// DESC		: 	Scriptable Object class for Audio Cue assets.
// -------------------------------------------------------

[CreateAssetMenu(fileName = "Audio Cue")]
public class AudioCue : ScriptableObject
{
    // Inspector Assigned
    [SerializeField] 					string				_mixerGroup		=	string.Empty;
    //[SerializeField]                  AudioMixerGroup     _audioGroup     =   null;   // TO DO: Change to AUDIO GROUP type instead of string?!
    //[SerializeField] [Range(0.0f,1.0f)]	float				_volume			=	1.0f;
    [SerializeField] [MinMaxRange(0, 1)] RangedFloat _volume = new RangedFloat(1, 1);
    [SerializeField] [MinMaxRange(0, 2)] RangedFloat _pitch = new RangedFloat(1,1);
    [SerializeField] [Range(0.0f,1.0f)] float				_spatialBlend	=	1.0f;
	[SerializeField] [Range(0, 256)]	int					_priority		=	128; 
	[SerializeField]					List<AudioClip>		_audioClips = 	new List<AudioClip>();


	// Public Properties
    public string mixerGroup	{ get {return _mixerGroup;}}
  	public float  volume		{ get{ return Random.Range(_volume.minValue, _volume.maxValue);}}
    public float  pitch         { get { return Random.Range(_pitch.minValue, _pitch.maxValue);}}
    public float  spatialBlend	{ get{ return _spatialBlend;}}
    public int	  priority		{ get{ return _priority;}}
    public List<AudioClip> audioClips { get { return _audioClips; } }


    // ---------------------------------------------------
    // Name	:	audioClip (Property)
    // Desc	:	Returns a random audio clip
    // --------------------------------------------------- 
    public AudioClip audioClip
    {
    	get
    	{
    		if (_audioClips.Count==0) return null;
            
    		AudioClip clip = _audioClips [Random.Range( 0, _audioClips.Count )];
    		return clip;
    	}
    }


    // preview button in inspector
    public void Play(AudioSource source)
    {
        if (_audioClips.Count == 0) return;

        source.clip = _audioClips[Random.Range(0, _audioClips.Count)];
        source.volume = Random.Range(_volume.minValue, _volume.maxValue);
        source.pitch = Random.Range(_pitch.minValue, _pitch.maxValue);
        source.Play();
    }


}
