﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.AI;

public class EnemyData
{
    // TO DO: Add enemy data structure here!  
    // (FOV, State, etc, etc)
    // Q: Which parts are fixed during gameplay (same as Prefab data), and which parts are dynamic, and changed based on state?
    // Need to Save & Load the dynamic data.
}


public class EnemyController : MonoBehaviour
{
    // references
    public Light myStatusLight;
    public Color32 lightColorIdle;
    public Color32 lightColorChase;
    private GameObject player;
    private NavMeshAgent agent;
    private NavMeshPath path;
    //private AudioSource audSrc;

    // fixed entity data
    public float rotateSpeed = 200f;
    public float moveSpeed = 10f;
    public float fieldOfView = 80f;
    public float viewDistance = 3f;
    public float timeUntilGiveUpChase = 1f;

    // dynamic state data
    public enum AIState { None, Idle, Chase };
    public AIState myAIState = AIState.None;
    public Vector3 lastSightedPosition = Vector3.zero;
    public float searchingTimer = 0f;
    private bool isAlerted = false;

    // private fields
    private RaycastHit hit;

    private AudioSource audSrc = null;
    public AudioClip audio_Alert;

    void Start()
    {
        // self references
        agent = GetComponent<NavMeshAgent>();
        myStatusLight = GetComponentInChildren<Light>();
        audSrc = GetComponentInChildren<AudioSource>();

        // set up references
        player = GameObject.FindGameObjectWithTag("Player");
        
        // create elements
        path = new NavMeshPath();
        
        // initialise state   
        // TO DO: Need to initialise from LOAD state, if loaded!!  ("if there is load data available, load that instead")
        myAIState = AIState.Idle;

    }


    void Update()
    {
        if (myAIState == AIState.Idle)
            Idle();
        if (myAIState == AIState.Chase)
            Chase();

    }


    void Idle()
    {
        // rotate ("scanning the area for the player")
        transform.RotateAround(transform.position, Vector3.up, rotateSpeed * Time.deltaTime);

        // if player is within viewDistance
        if (Vector3.Distance(player.transform.position, transform.position) < viewDistance)
        {
            // if player is within fieldOfView
            if (Vector3.Angle(player.transform.position - transform.position, transform.forward) <= fieldOfView / 2)
            {
                // raycast to see if I have line of sight
                if (Physics.Linecast(transform.position, player.transform.position, out hit))
                {
                    //Debug.Log("Hit collider:  " + hit.collider.name);
                    if (hit.collider.name == "Player")
                    {
                        lastSightedPosition = player.transform.position;
                        searchingTimer = 0f;
                        myAIState = AIState.Chase;
                        myStatusLight.color = lightColorChase;
                        if (!isAlerted)
                        {
                            isAlerted = true;
                            audSrc.PlayOneShot(audio_Alert);
                        }
                    }
                }
            }
        }
    }


    void Chase()
    {
        // if player is within viewDistance
        if (Vector3.Distance(player.transform.position, transform.position) < viewDistance)
        {
            // if player is within fieldOfView
            if (Vector3.Angle(player.transform.position - transform.position, transform.forward) <= fieldOfView / 2)
            {
                // raycast to see if I have line of sight
                if (Physics.Linecast(transform.position, player.transform.position, out hit))
                {
                    if (hit.collider.name == "Player")
                    {
                        lastSightedPosition = player.transform.position;
                        searchingTimer = 0f;
                        myAIState = AIState.Chase;
                    }
                }
            }
        }
        
        
        // navigate to the lastSightedPosition
        agent.CalculatePath(lastSightedPosition, path);
        if (path.status == NavMeshPathStatus.PathComplete || true)
            agent.destination = lastSightedPosition;


        // if near lastSightedLocation, count up searchingTimer  (searching for player)
        if (Vector3.Distance(lastSightedPosition, transform.position) < 3f)
        {
            searchingTimer += Time.deltaTime;
        }

        if (searchingTimer > timeUntilGiveUpChase)
        {
            myAIState = AIState.Idle;
            myStatusLight.color = lightColorIdle;
            isAlerted = false;
        }
        

    }
    

}