﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PlayerController : MonoBehaviour
{
    public float moveSpeed = 5f;
    public float turnSpeed = 200f;
    public float gravityAmount = 10f;

    private Vector3 moveVector = Vector3.zero;
    private Vector3 rotateVector = Vector3.zero;

    private CharacterController characterController = null;
    public LevelManager level = null;

    private AudioSource audSrc = null;
    public AudioClip audio_Pickup;
    public AudioClip audio_Die;
    

    void Start()
    {
        // cache references
        characterController = GetComponent<CharacterController>();
        audSrc = GetComponent<AudioSource>();
    }
    

    void Update()
    {
        ProcessMovement();
    }
    

    void ProcessMovement()
    {
        // get input
        float horizontal = Input.GetAxis("Horizontal");
        float vertical = Input.GetAxis("Vertical");

        // set move and rotate vectors
        moveVector = new Vector3(0, -gravityAmount * Time.deltaTime, vertical * moveSpeed * Time.deltaTime);
        moveVector = transform.TransformDirection(moveVector);
        rotateVector = new Vector3(0, horizontal * turnSpeed * Time.deltaTime, 0);
        
        // apply movement & rotation
        characterController.Move(moveVector);
        transform.Rotate(rotateVector);
    }


    void OnTriggerEnter(Collider other)
    {
        if (other.gameObject.GetComponent<Gem>())
        {
            level.CollectGem();
            other.gameObject.SetActive(false);
            audSrc.PlayOneShot(audio_Pickup);
            level.CheckForWinState();
        }
    }


}
